"""

Revision ID: 6e324523861c
Revises: ea815a161929
Create Date: 2025-06-21 17:08:08.289299

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa

import sqlmodel

# revision identifiers, used by Alembic.
revision: str = "6e324523861c"
down_revision: Union[str, None] = "ea815a161929"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("subject", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "symbol_without_suffix",
                sqlmodel.sql.sqltypes.AutoString(),
                nullable=True,
            )
        )

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("subject", schema=None) as batch_op:
        batch_op.drop_column("symbol_without_suffix")

    # ### end Alembic commands ###
